@@ -1,12 +1,12 @@
 // Copyright 2020-2021 the Deno authors. All rights reserved. MIT license.
 use super::{Context, LintRule};
 use crate::handler::{Handler, Traverse};
-use crate::scopes::Scope;
 use crate::{Program, ProgramRef};
 use deno_ast::swc::common::Spanned;
 use deno_ast::swc::utils::ident::IdentLike;
 use deno_ast::view as ast_view;
 use deno_ast::view::NodeTrait;
+use deno_ast::Scope;
 use if_chain::if_chain;
 use std::sync::Arc;
 
@@ -136,12 +136,26 @@ impl Handler for PreferPrimordialsHandler {
     }
   }
 
+  fn expr_or_spread(
+    &mut self,
+    expr_or_spread: &ast_view::ExprOrSpread,
+    ctx: &mut Context,
+  ) {
+    if_chain! {
+      if expr_or_spread.inner.spread.is_some();
+      if !expr_or_spread.inner.expr.is_new();
+      then {
+        ctx.add_diagnostic_with_hint(expr_or_spread.span(), CODE, MESSAGE, HINT);
+      }
+    }
+  }
+
   fn member_expr(
     &mut self,
     member_expr: &ast_view::MemberExpr,
     ctx: &mut Context,
   ) {
-    use deno_ast::view::Expr;
+    use ast_view::Expr;
 
     // If `member_expr.obj` is an array literal, access to its properties or
     // methods should be replaced with the one from `primordials`.
@@ -273,6 +287,21 @@ const parseInt = () => {};
 parseInt();
       "#,
       r#"const foo = { Error: 1 };"#,
+      r#"
+const { SafeArrayIterator } = primordials;
+[1, 2, ...new SafeArrayIterator(arr)];
+foo(1, 2, ...new SafeArrayIterator(arr));
+new Foo(1, 2, ...new SafeArrayIterator(arr));
+      "#,
+      r#"
+const { SafeArrayIterator } = primordials;
+[1, 2, ...new SafeArrayIterator([1, 2, 3])];
+foo(1, 2, ...new SafeArrayIterator([1, 2, 3]));
+new Foo(1, 2, ...new SafeArrayIterator([1, 2, 3]));
+      "#,
+      r#"
+({ ...{} });
+      "#,
     };
   }
 
@@ -435,6 +464,48 @@ const noop = Function.prototype;
           hint: HINT,
         },
       ],
+      r#"[1, 2, ...arr];"#: [
+        {
+          col: 7,
+          message: MESSAGE,
+          hint: HINT,
+        },
+      ],
+      r#"foo(1, 2, ...arr);"#: [
+        {
+          col: 10,
+          message: MESSAGE,
+          hint: HINT,
+        },
+      ],
+      r#"new Foo(1, 2, ...arr);"#: [
+        {
+          col: 14,
+          message: MESSAGE,
+          hint: HINT,
+        },
+      ],
+      r#"[1, 2, ...[3]];"#: [
+        {
+          col: 7,
+          message: MESSAGE,
+          hint: HINT,
+        },
+      ],
+      r#"foo(1, 2, ...[3]);"#: [
+        {
+          col: 10,
+          message: MESSAGE,
+          hint: HINT,
+        },
+      ],
+      r#"new Foo(1, 2, ...[3]);"#: [
+        {
+          col: 14,
+          message: MESSAGE,
+          hint: HINT,
+        },
+      ],
     }
   }
 }
